using Toybox.Graphics as Gfx;
using Toybox.WatchUi as Ui;
using Toybox.Time;
using Toybox.Time.Gregorian as Cal;

using C as C;

class Moon
{

var phase;

var year;
var month;
var day;

var x;
var y;

private var mo_cumm = [ 0,   0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334, 365 ];

private var icons;
private var icons_sm = [ Rez.Drawables.moon0sm,
                         Rez.Drawables.moon1sm,
                         Rez.Drawables.moon2sm,
                         Rez.Drawables.moon3sm,
                         Rez.Drawables.moon4sm,
                         Rez.Drawables.moon5sm,
                         Rez.Drawables.moon6sm,
                         Rez.Drawables.moon7sm ];

private var icons_big = [ Rez.Drawables.moon0,
                          Rez.Drawables.moon1,
                          Rez.Drawables.moon2,
                          Rez.Drawables.moon3,
                          Rez.Drawables.moon4,
                          Rez.Drawables.moon5,
                          Rez.Drawables.moon6,
                      Rez.Drawables.moon7 ];

public function initialize()
{

    x = C.center_x + (C.center_x * 0.45); 
    y = C.center_y + (C.center_y * 0.3); 

    if (C.radius < 150) {
        icons = icons_sm;
    } else {
        icons = icons_big;
    }

    year = 0;
    get_phase();
}

public function show(dc)
{

    var bits = new Ui.Bitmap({ :rezId => get_phase(),
                               :locX => x,
                               :locY => y });
    bits.draw(dc);
}

function epoch_days(d)
{

    var yr = d.year;
    if (d.month < 2) {
        yr = yr - 1;
    }
    var leaps = (yr/4).toNumber() - (yr/100).toNumber() + (yr/400).toNumber();

    var days = (d.year * 365) + d.day + mo_cumm[d.month];
    return days + leaps;
}

function moon_day(d)
{

    //
    //  Should return (d + 1) % 29.530588853 but Monkey C
    //  doesn't know how to do a floating mod.  We'll do
    //  the % operation in 2 calculations instead.
    //
    d = d + 1;
    var n = (d / 29.530588853).toNumber();
    return (d - (n * 29.530588853)).toNumber();
}

function moon_icon(mday)
{

    mday = mday + 2;
    var d = moon_day(mday);
    var cycle = (moon_day(mday + 29) == d) ? 29 : 30;
    return ((d * 8) / cycle).toNumber();
}

function get_phase()
{

    var t = Cal.info(Time.now(), Time.FORMAT_SHORT);
    if (t.year == year && t.month == month && t.day == day) {
        return phase;
    }
    year = t.year;
    month = t.month;
    day = t.day;

    var ref = Cal.moment({:year  => 2000,
                          :month => 1,
                          :day   => 6,
                          :hour  => 12 });
    var info = Cal.info(ref, Time.FORMAT_SHORT);
    var ndays = epoch_days(t) - epoch_days(info);
    var idx = moon_icon(ndays);
    phase = icons[idx];
    return phase;
}

}
